@@ -22,7 +22,7 @@
 #define HAVE_ICONV_H
 #define HAVE_PNG_H
 #define HAVE_LIBPNG
-#define HAVE_SNDFILE
+#undef HAVE_SNDFILE
 #undef HAVE_PORTAUDIO
 #undef HAVE_SLES_OPENSLES_H
 #undef HAVE_LIBFFTW3
@@ -31,7 +31,7 @@
 //#define HAVE_FTDI_NEW
 #define HAVE_FTDI_USB_GET_STRINGS
 #define HAVE_SYS_SELECT_H
-#define HAVE_MATH_H
+//#define HAVE_MATH_H
 #define HAVE_UNISTD_H
 #define HAVE_TERMIOS_H
 #define HAVE_SYS_WAIT_H
@@ -41,6 +41,7 @@
 
 #include "regex_.h"
 
+int android_main(int argc, char *argv[]);
 void android_restore_state(void);
 void android_update_package(char *filename);
 
