@@ -66,7 +66,10 @@ struct dsp{
 #ifdef HAVE_PORTAUDIO
     PaStream *pa;
     PaStreamParameters pa_params;
-	int pa_play, pa_rec;
+    int pa_play, pa_rec;
+    int pa_play_hi, pa_rec_hi;
+    gchar *pa_play_s, *pa_rec_s;
+
 #endif 
 
 /* OpenSL ES only */
