@@ -1121,7 +1121,8 @@ destroy:;
             maps_reload();
 			maps_update_showwwls();
 #ifdef Z_HAVE_SDL
-			rain_reload();
+            int wwlchanged = strcmp(cfg->pwwlo, ctest->pwwlo);
+            if (wwlchanged) rain_reload();
 #endif
         }
         
