@@ -494,8 +494,10 @@ void sw_map_redraw_rain(struct subwin *sw, SDL_Rect *aarea, int flags){
 		km2px_d(sw, scp->kx + grain->minscpdist, scp->ky, &px2, &py2);
 		int r = (int)sqrt((double)(px1 - px2)*(px1 - px2) + (py1 - py2)*(py1 - py2));
 		int c = scp->qrb <= grain->maxqrb ? sdl->gr[15] : sdl->gr[5];
-		z_circle(sw->screen, px1, py1, r, c);
-		if (r > 3) z_circle(sw->screen, px1, py1, r - 2, c);
+        if (!grain->debug) {
+            z_circle(sw->screen, px1, py1, r, c);
+            if (r > 3) z_circle(sw->screen, px1, py1, r - 2, c);
+        }
 	}
 }
 
@@ -509,8 +511,7 @@ void plot_rain_provider(struct subwin *sw, SDL_Rect *aarea, struct rain_provider
 	int kx, ky;
     int px1, px2, px3 = 0, px4 = 0, py1, py2, py3 = 0, py4 = 0;
 
-    if (provider->img == NULL) return;
-	if (provider->w <= 0) return;
+	//if (provider->w <= 0) return;
     
 
 	area.x = aarea->x - 16; // maximal "pixel" size is about 16x16
@@ -524,21 +525,42 @@ void plot_rain_provider(struct subwin *sw, SDL_Rect *aarea, struct rain_provider
 
 	if (provider->rain->debug){
 		// debug mode - only pixels
-		for (iy = 0; iy < provider->img->h; iy++){
-			for (ix = 0; ix < provider->img->w; ix++){
-				c = z_getpixel_fmt(provider->img, ix, iy, sdl->screen->format);
-				//if (c == 0x00aad3df) continue;
+        if (provider->tiles) {
+            for (i = 0; i < provider->tiles->len; i++) {
+                struct map_tile *tile = (struct map_tile*)g_ptr_array_index(provider->tiles, i);
+				if (tile->img == NULL) continue;
 
-				
-				provider->img2hw(provider, provider->rv_px0 + ix, provider->rv_py0 + iy, &hh1, &ww1);
+                for (iy = 0; iy < tile->img->h; iy++) {
+                    for (ix = 0; ix < tile->img->w; ix++) {
+                        c = z_getpixel_fmt(tile->img, ix, iy, sdl->screen->format);
 
-				hw2km(gses->myh, gses->myw, hh1, ww1, &kx, &ky);
-				km2px(sw, kx, ky, &px1, &py1);
-        
-				if (!z_overlapped_pixel(aarea, px1, py1)) continue;
-				z_putpixel(sw->screen, px1, py1, c);
+                        provider->img2hw(provider, tile, ix, iy, &hh1, &ww1);
+
+                        hw2km(gses->myh, gses->myw, hh1, ww1, &kx, &ky);
+                        km2px(sw, kx, ky, &px1, &py1);
+
+                        if (!z_overlapped_pixel(aarea, px1, py1)) continue;
+                        z_putpixel(sw->screen, px1, py1, c);
+                    }
+                }
 			}
-		}
+        }else if (provider->img != NULL) {
+            for (iy = 0; iy < provider->img->h; iy++) {
+                for (ix = 0; ix < provider->img->w; ix++) {
+                    c = z_getpixel_fmt(provider->img, ix, iy, sdl->screen->format);
+                    //if (c == 0x00aad3df) continue;
+
+
+                    provider->img2hw(provider, NULL, ix, iy, &hh1, &ww1);
+
+                    hw2km(gses->myh, gses->myw, hh1, ww1, &kx, &ky);
+                    km2px(sw, kx, ky, &px1, &py1);
+
+                    if (!z_overlapped_pixel(aarea, px1, py1)) continue;
+                    z_putpixel(sw->screen, px1, py1, c);
+                }
+            }
+        }
 	}else{
 		// not debug
 		SDL_SetClipRect(sw->screen, aarea); // not area
@@ -578,7 +600,7 @@ void plot_rain_provider(struct subwin *sw, SDL_Rect *aarea, struct rain_provider
 			}
 		}
 
-		if (ci == 0){
+		if (ci == 0 && provider->img != NULL){
 			// borders
 			for (iy = provider->top; iy <= provider->img->h && iy <= provider->bottom + 1; iy++){
 				ww1 = provider->w1 + (iy - provider->wpx1) * provider->wmult;
