@@ -312,12 +312,12 @@ static void really_close_ctest(void)
 	map_recalc_cors();
 	maps_reload();
 	maps_update_showwwls();
-	rain_reload();
+	//rain_reload(); NE, vol se u z free_ctest
 	beacondb_recalc_qrbqtf(gbeacondb);
 #endif   
 	chart_reload();
 #ifdef OCT8TOR
-	oct8tor_send_band(goct8tor, NULL);
+	oct8tor_send_band(goct8tor, "*");
 #endif
 }
 
