@@ -149,16 +149,16 @@ void menu_rain_opts(void *itdata, void *menudata){
     d->items[i].gnum = 1;
     d->items[i].dlen = sizeof(int);
     d->items[i].data = (char *)&rain_chmi;
-    d->items[i].msg = "CHMI";
+    d->items[i].msg = "CHMI (legacy old web)";
     d->items[i].wrap = 1;
 
-	d->items[++i].type = D_CHECKBOX;      
-    d->items[i].gid = 0;
-    d->items[i].gnum = 1;
-    d->items[i].dlen = sizeof(int);
-    d->items[i].data = (char *)&rain_weatheronline;
-    d->items[i].msg = "Weatheronline (EU)";
-    d->items[i].wrap = 1;
+	//d->items[++i].type = D_CHECKBOX;      
+ //   d->items[i].gid = 0;
+ //   d->items[i].gnum = 1;
+ //   d->items[i].dlen = sizeof(int);
+ //   d->items[i].data = (char *)&rain_weatheronline;
+ //   d->items[i].msg = "Weatheronline (EU)";
+ //   d->items[i].wrap = 1;
 
 	
 	d->items[i].wrap++;
