@@ -34,7 +34,8 @@ struct oct8tor{
 	int keepalive;
 
 	bool onconnect;
-	char *adifband;
+	char* adifband;
+	char* currentadifband; // main thread
 };
 
 struct oct8tor *init_oct8tor(void);
@@ -50,7 +51,8 @@ void oct8tor_read_handler(int n, char **line);
 void oct8tor_onreceived(struct oct8tor *oct, struct zjson *line);
 
 void oct8tor_rcvd_rotars(struct oct8tor *oct, struct zjson *json);
-void oct8tor_rcvd_angle(struct oct8tor *oct, struct zjson *json);
+void oct8tor_rcvd_angle(struct oct8tor* oct, struct zjson* json);
+void oct8tor_rcvd_aangle(struct oct8tor *oct, struct zjson *json);
 
 
 void oct8tor_send_band(struct oct8tor *oct8tor, const char *adifband);
