@@ -1326,6 +1326,7 @@ void free_ctest(){
 
     if (!ctest) return;
 
+    int wwlchanged = strcmp(cfg->pwwlo, ctest->pwwlo);
 
     zselect_timer_kill(zsel, ctest->minute_timer_id);
     if (gses->skedw_timer_id > 0) {
@@ -1397,7 +1398,7 @@ void free_ctest(){
 	maps_update_showwwls();
     chart_reload();
 #ifdef Z_HAVE_SDL
-	rain_reload();
+	if (wwlchanged) rain_reload();
 #endif
 }
 
