@@ -40,21 +40,21 @@ struct rain *init_rain(){
 	rain->debug = cfg->rain_debug;
 
 	rain->colors = g_new0(int, RAIN_COLORS);
-	rain->colors[0] = z_makecol(56, 0, 112);
-	rain->colors[1] = z_makecol(48, 0, 168);     // 0.1 mm/h
-	rain->colors[2] = z_makecol(0, 0, 252);
-	rain->colors[3] = z_makecol(0, 108, 192);
-	rain->colors[4] = z_makecol(0, 160, 0);
-	rain->colors[5] = z_makecol(0, 188, 0);	     // 1 mm/h
-	rain->colors[6] = z_makecol(52, 216, 0);
-	rain->colors[7] = z_makecol(156, 220, 0);
-	rain->colors[8] = z_makecol(224, 220, 0);
-	rain->colors[9] = z_makecol(252, 176, 0);    // 10 mm/h
-	rain->colors[10] = z_makecol(252, 132, 0);
-	rain->colors[11] = z_makecol(252, 88, 0);
-	rain->colors[12] = z_makecol(252, 0, 0);
-	rain->colors[13] = z_makecol(160, 0, 0);     // 100 mm/h
-	rain->colors[14] = z_makecol(252, 252, 252);
+	rain->colors[0] = z_makecol(56, 0, 112);     //  4 dBZ   
+	rain->colors[1] = z_makecol(48, 0, 168);     //  8 dBZ   0.1 mm/h
+	rain->colors[2] = z_makecol(0, 0, 252);      // 12 dBZ
+	rain->colors[3] = z_makecol(0, 108, 192);    // 16 dBZ
+	rain->colors[4] = z_makecol(0, 160, 0);      // 20 dBZ
+	rain->colors[5] = z_makecol(0, 188, 0);      // 24 dBZ	   1 mm/h
+	rain->colors[6] = z_makecol(52, 216, 0);     // 28 dBZ
+	rain->colors[7] = z_makecol(156, 220, 0);    // 32 dBZ
+	rain->colors[8] = z_makecol(224, 220, 0);    // 36 dBZ
+	rain->colors[9] = z_makecol(252, 176, 0);    // 40 dBZ    10 mm/h
+	rain->colors[10] = z_makecol(252, 132, 0);   // 44 dBZ
+	rain->colors[11] = z_makecol(252, 88, 0);    // 48 dBZ
+	rain->colors[12] = z_makecol(252, 0, 0);     // 52 dBZ
+	rain->colors[13] = z_makecol(160, 0, 0);     // 56 dBZ   100 mm/h
+	rain->colors[14] = z_makecol(252, 252, 252); // 60 dBZ
 
 	rain->bwcolors = g_new0(int, RAIN_COLORS);
     for (i = 0; i < RAIN_COLORS; i++){
@@ -176,7 +176,7 @@ struct rain *init_rain(){
 		rain->chmi = provider;
 	}
 
-	if (cfg->rain_weatheronline){
+	if (0 && cfg->rain_weatheronline){ // DISABLED
 		struct rain_provider *provider = g_new0(struct rain_provider, 1);
 		provider->rain = rain;
 		provider->name = "weatheronline";
@@ -355,9 +355,12 @@ void free_rain_provider(struct rain_provider *provider){
 	g_free(provider->srccols);
 	g_free(provider->translation);
 	g_free(provider->title);
+	g_free(provider->rv_host);
 	g_free(provider->rv_path);
 	g_free(provider->rv_oldpath);
+	zg_ptr_array_free_all(provider->tiles);
 	g_free(provider);
+	if (provider->rv_pal) g_hash_table_destroy(provider->rv_pal);
 	dbg("freed rainviewer %p\n", provider);
 
 }
@@ -384,15 +387,15 @@ static void rain_truncate_hash(gpointer key, gpointer value, gpointer user_data)
 }
 
 
-static void rain_provider_add(struct rain_provider *provider, int ix, int iy, int ci){
+static void rain_provider_add(struct rain_provider *provider, struct map_tile *tile, int ix, int iy, int ci){
 	double hh1, ww1, hh2, ww2;
 	gpointer key;
 	GHashTable *hash;
 	struct zbinbuf *zbb;
 	struct rain_km km;
 
-    provider->img2hw(provider, ix - 1, iy - 1, &hh1, &ww1); 
-    provider->img2hw(provider, ix + 1, iy + 1, &hh2, &ww2);// when +1 space between bars 
+    provider->img2hw(provider, tile, ix - 1, iy - 1, &hh1, &ww1); 
+    provider->img2hw(provider, tile, ix + 1, iy + 1, &hh2, &ww2);// when +1 space between bars 
 
 	//hh2 = hh1 + provider->hmult;
 	//ww2 = ww1 + provider->wmult;
@@ -446,7 +449,7 @@ void rain_provider_draw_colors(struct rain_provider *provider, SDL_Surface *surf
 	}
 }
 
-void rain_linear_img2hw(struct rain_provider *provider, float ix, float iy, double *h, double *w){
+void rain_linear_img2hw(struct rain_provider *provider, struct map_tile* tile, int ix, int iy, double *h, double *w){
 	*h = provider->h1 + (ix - provider->hpx1) * provider->hmult;
 	*w = provider->w1 + (iy - provider->wpx1) * provider->wmult;
 }
@@ -586,8 +589,8 @@ void rain_meteox_load(struct rain_provider *provider){
 	SDL_Surface *img = provider->img;
 	Uint8 r, g, b, a;
 
-	provider->w = img->w;
-	provider->h = img->h;
+	//provider->w = img->w;
+	//provider->h = img->h;
 
 	for (i = 0; i < RAIN_COLORS; i++) {
 		GHashTable *hash = provider->hashes[i];
@@ -643,7 +646,7 @@ void rain_meteox_load(struct rain_provider *provider){
 			ci = provider->translation[ci];
 			if (ci < 5) continue;
             if (provider->rain->debug) z_putpixel_fmt(img, ix, iy, provider->rain->colors[ci], sdl->screen->format);
-			rain_provider_add(provider, ix, iy, ci);
+			rain_provider_add(provider, NULL, ix, iy, ci);
 		}
 	}
 }
@@ -759,8 +762,8 @@ void rain_wetteronline_load(struct rain_provider *provider){
 	provider->translation[4] = 11;
 	provider->translation[5] = 12;
 	
-	provider->w = img->w;
-	provider->h = img->h;
+	//provider->w = img->w;
+	//provider->h = img->h;
 	
 	for (i = 0; i < RAIN_COLORS; i++) {
 		GHashTable *hash = provider->hashes[i];
@@ -783,7 +786,7 @@ void rain_wetteronline_load(struct rain_provider *provider){
 					ci = provider->translation[ci];
 					if (ci < 5) break;
 
-					rain_provider_add(provider, ix, iy, ci);
+					rain_provider_add(provider, NULL, ix, iy, ci);
 					if (provider->rain->debug) z_putpixel_fmt(img, ix, iy, provider->rain->colors[ci], sdl->screen->format); 
 					break;
 				}
@@ -823,10 +826,10 @@ void rain_chmi_timer(void *arg){
 
     //dbg("rain_timer %d.%d.%d %d.%02d.%02d\n", tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec); 
 
-    url = g_strdup_printf("https://www.chmi.cz/files/portal/docs/meteo/rad/inca-cz/data/czrad-z_max3d/pacz2gmaps3.z_max3d.%04d%02d%02d.%02d%02d.0.png", 
+    url = g_strdup_printf("https://intranet.chmi.cz/files/portal/docs/meteo/rad/inca-cz/data/czrad-z_max3d/pacz2gmaps3.z_max3d.%04d%02d%02d.%02d%02d.0.png", 
         tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,
 		tm.tm_hour, tm.tm_min); 
-    //url = g_strdup_printf("http://portal.chmi.cz/files/portal/docs/meteo/rad/inca-cz/data/czrad-z_max3d/pacz2gmaps3.z_max3d.20160708.1800.0.png");
+    //url = g_strdup_printf("http://porta.chmi.cz/files/portal/docs/meteo/rad/inca-cz/data/czrad-z_max3d/pacz2gmaps3.z_max3d.20160708.1800.0.png");
 
 	provider->http = zhttp_init();
     dbg("CHMI url = '%s'\n", url);
@@ -894,8 +897,8 @@ void rain_chmi_load(struct rain_provider *provider){
 	SDL_Surface *img = provider->img;
 	int red = SDL_MapRGB(sdl->screen->format, 252, 0, 0);
 	
-	provider->w = img->w;
-	provider->h = img->h;
+	//provider->w = img->w;
+	//provider->h = img->h;
 
 	for (i = 0; i < RAIN_COLORS; i++) {
 		GHashTable *hash = provider->hashes[i];
@@ -914,7 +917,7 @@ void rain_chmi_load(struct rain_provider *provider){
 			for (ci = 0; ci < RAIN_COLORS; ci++){
 				if (c == provider->rain->colors[ci]){
 					if (ci < 5) break;
-					rain_provider_add(provider, ix, iy, ci/*9*/);
+					rain_provider_add(provider, NULL, ix, iy, ci/*9*/);
 					break;
 				}
 			}
@@ -1041,8 +1044,8 @@ void rain_weatheronline_load(struct rain_provider *provider){
 	provider->translation[11] = 13; // 100 mm/h
 	provider->translation[12] =	14;
 	
-	provider->w = img->w;
-	provider->h = img->h;
+	//provider->w = img->w;
+	//provider->h = img->h;
 	
 	for (i = 0; i < RAIN_COLORS; i++) {
 		GHashTable *hash = provider->hashes[i];
@@ -1080,7 +1083,7 @@ void rain_weatheronline_load(struct rain_provider *provider){
 					ci = provider->translation[ci];
 					if (ci < 5) break;
 
-					rain_provider_add(provider, ix, iy, ci);
+					rain_provider_add(provider, NULL, ix, iy, ci);
 					if (provider->rain->debug) z_putpixel_fmt(img, ix, iy, provider->rain->colors[ci], sdl->screen->format); 
 					break;
 				}
@@ -1110,7 +1113,7 @@ gpointer rain_weatheronline_hash(short kx, short ky, int ci){
 }
 
 
-void rain_weatheronline_img2hw(struct rain_provider *provider, float ix, float iy, double *h, double *w){
+void rain_weatheronline_img2hw(struct rain_provider *provider, struct map_tile* tile, int ix, int iy, double *h, double *w){
 	double dx = ix - 234;
 	double dy = iy + 280;
 	double d = sqrt(dx * dx + dy * dy) / 12.6 - 4.75;
@@ -1123,45 +1126,102 @@ void rain_weatheronline_img2hw(struct rain_provider *provider, float ix, float i
 	*h *= M_PI / 180.0;
 }
 
+/* Mercator tiles */
+
+int tiles_lon2tileX(double lon, int zoom) {
+	int n = 1 << zoom;
+	int ix = (int)((lon + 180.0) / 360.0 * n);
+	if (ix < 0) ix = 0;
+	if (ix >= n) ix = n - 1;
+	return ix;
+}
+
+int tiles_lat2tileY(double lat, int zoom) {
+	double lat_rad = lat * M_PI / 180.0;
+	int n = 1 << zoom;
+	int iy = (int)((1.0 - log(tan(lat_rad) + 1.0 / cos(lat_rad)) / M_PI) / 2.0 * n);
+	if (iy < 0) iy = 0;
+	if (iy >= n) iy = n - 1;
+	return iy;
+}
+
+double tiles_tileX2lon(int tileX, int iy, int zoom) {
+	int n = 1 << zoom;
+	double x = (tileX * 256.0 + iy) / (256.0 * n);
+	return x * 360.0 - 180.0;
+}
+
+double tiles_tileY2lat(int tileY, int iy, int zoom) {
+	int n = 1 << zoom;
+	double y = 1.0 - 2.0 * (tileY * 256.0 + iy) / (256.0 * n);
+	double lat_rad = atan(sinh(M_PI * y));
+	return lat_rad * 180.0 / M_PI;
+}
+
+
+
+
+
 /* www.rainviewer.com */
 void rain_rainviewer_init(struct rain_provider *provider){
 	if (gses == NULL || provider == NULL) return;
 
 	provider->rv_size = 256;
-	double qrb = cfg->rain_maxqrb; // TODO
-	double qtf;
-	double maxh = 0, maxw = 0;
-	for (qtf = 0.0; qtf <= 180.0; qtf += 30.0){
-		double h, w;
-		qrbqtf2hw(/*gses->myh*/ 0, gses->myw, qrb, qtf * M_PI / 180.0, &h, &w);
-		double dw = fabs(w - gses->myw);
-		if (h > maxh) maxh = h;
-		if (dw > maxw) maxw = dw;
-		//dbg("h=%3.1f  w=%3.1f\n", h * 180.0 / M_PI, w * 180 / M_PI);
+	provider->tiles = g_ptr_array_new();
+
+	double qrb = cfg->rain_maxqrb;
+	double minlon = +400.0, maxlon = -400, minlat = +400, maxlat = -400;
+	for (double qtf_deg = 0.0; qtf_deg < 360.0; qtf_deg += 30.0){
+		double lon, lat;
+		qrbqtf2lonlat(gses->myh, gses->myw, qrb, qtf_deg, &lon, &lat);
+		//dbg("qrb=%1.0f  lon=%2.1f  lat=%3.1f\n", qrb, lon, lat);
+
+		if (lon < minlon) minlon = lon;
+		if (lon > maxlon) maxlon = lon;
+		if (lat < minlat) minlat = lat;
+		if (lat > maxlat) maxlat = lat;
 	}
-	dbg("qrb=%1.0f  maxh=%3.1f  maxw=%3.1f\n", qrb, maxh * 180.0 / M_PI, maxw * 180 / M_PI);
+	dbg("qrb=%1.0f  lon=%2.1f..%2.1f  lat=%3.1f..%3.1f\n", qrb, minlon, maxlon, minlat, maxlat);
 
-	provider->rv_zoom = tile_googlezoom(2 * maxh, 2 * maxw);
-	provider->rv_pow = pow(2, provider->rv_zoom);
-	provider->rv_mul = provider->rv_size * provider->rv_pow;
-	dbg("zoom=%d  pow=%d  mul=%d\n", provider->rv_zoom, provider->rv_pow, provider->rv_mul);
-	//provider->rv_zoom = 0;
-
-	rain_rainviewer_hw2himg(provider, gses->myh, gses->myw, &provider->rv_mypx, &provider->rv_mypy);
-	dbg("mypx=%d  mypy=%d\n", provider->rv_mypx, provider->rv_mypy);
-	provider->rv_px0 = provider->rv_mypx - (provider->rv_size -1) / 2.0;
-	provider->rv_py0 = provider->rv_mypy - (provider->rv_size - 1) / 2.0;
-	dbg("px0=%d  py0=%d\n", provider->rv_px0, provider->rv_py0);
+	provider->rv_zoom = tile_rainviewer_zoom(maxlon-minlon, maxlat-minlat);
 
 
-	double h, w;
+	int rv_mintilex = tiles_lon2tileX(minlon, provider->rv_zoom);
+	int rv_maxtilex = tiles_lon2tileX(maxlon, provider->rv_zoom);
+	int rv_mintiley = tiles_lat2tileY(maxlat, provider->rv_zoom); // y=0 je severn pl
+	int rv_maxtiley = tiles_lat2tileY(minlat, provider->rv_zoom);
+	dbg("tilex=%d..%d  lat=%d..%d\n", rv_mintilex, rv_maxtilex, rv_mintiley, rv_maxtiley);
+	for (int x = rv_mintilex; x <= rv_maxtilex; x++) {
+		for (int y = rv_mintiley; y <= rv_maxtiley; y++) {
+			struct map_tile* tile = g_new0(struct map_tile, 1);
+			tile->x = x;
+			tile->y = y;
+			g_snprintf(tile->url, sizeof(tile->url), "/%d/%d/%d/%d/0/0_0_0_0.png", provider->rv_size, provider->rv_zoom, x, y);
+			g_ptr_array_add(provider->tiles, tile);
+		}
+	}
+
+	//provider->rv_pow = pow(2, provider->rv_zoom);
+	//provider->rv_mul = provider->rv_size * provider->rv_pow;
+	//dbg("zoom=%d  pow=%d  mul=%d\n", provider->rv_zoom, provider->rv_pow, provider->rv_mul);
+	////provider->rv_zoom = 0;
+
+	//rain_rainviewer_hw2himg(provider, gses->myh, gses->myw, &provider->rv_mypx, &provider->rv_mypy);
+	//dbg("mypx=%d  mypy=%d\n", provider->rv_mypx, provider->rv_mypy);
+	//provider->rv_px0 = provider->rv_mypx - (provider->rv_size -1) / 2.0;
+	//provider->rv_py0 = provider->rv_mypy - (provider->rv_size - 1) / 2.0;
+	//dbg("px0=%d  py0=%d\n", provider->rv_px0, provider->rv_py0);
+	provider->tileidx = 0;
+
+
+	/*double h, w;
 	rain_rainviewer_img2hw(provider, 0, 0, &h, &w);
 	dbg("0 0 = %3.2f %3.2f\n", h * 180.0 / M_PI, w * 180.0 / M_PI);
 	rain_rainviewer_img2hw(provider, 128 * provider->rv_pow, 128 * provider->rv_pow, &h, &w);
 	dbg("128 128 = %3.2f %3.2f\n", h * 180.0 / M_PI, w * 180.0 / M_PI);
 	rain_rainviewer_img2hw(provider, 256 * provider->rv_pow, 256 * provider->rv_pow, &h, &w);
 	dbg("256 256 = %3.2f %3.2f\n", h * 180.0 / M_PI, w * 180.0 / M_PI);
-	dbg("\n");
+	dbg("\n");*/
 
 	/*rain_rainviewer_img2hw(provider, 4352, 2560, &h, &w);
 	dbg("4352 2560 = %3.2f %3.2f\n", h * 180.0 / M_PI, w * 180.0 / M_PI);
@@ -1181,55 +1241,138 @@ void rain_rainviewer_init(struct rain_provider *provider){
 	h *= 180 / M_PI;
 	w *= 180 / M_PI;*/
 
-	provider->hpx1 = provider->rv_px0;  // x
-	provider->hpx2 = provider->rv_px0 + provider->rv_size - 1; // x
-	provider->wpx1 = provider->rv_py0 + provider->rv_size - 1; // y
-	provider->wpx2 = provider->rv_py0; // y
+	//provider->hpx1 = provider->rv_px0;  // x
+	//provider->hpx2 = provider->rv_px0 + provider->rv_size - 1; // x
+	//provider->wpx1 = provider->rv_py0 + provider->rv_size - 1; // y
+	//provider->wpx2 = provider->rv_py0; // y
 
-	provider->left = provider->rv_px0;
-	provider->top = provider->rv_py0;
-	provider->right = provider->rv_px0 + provider->rv_size - 1;
-	provider->bottom = provider->rv_py0 + provider->rv_size - 1;
+	//provider->left = provider->rv_px0;
+	//provider->top = provider->rv_py0;
+	//provider->right = provider->rv_px0 + provider->rv_size - 1;
+	//provider->bottom = provider->rv_py0 + provider->rv_size - 1;
 
 
-	rain_rainviewer_img2hw(provider, provider->hpx1, provider->wpx1+1, &provider->h1, &provider->w1);
-	rain_rainviewer_img2hw(provider, provider->hpx2+1, provider->wpx2, &provider->h2, &provider->w2);
-	dbg("dh = %3.2f,  dw = %3.2f \n", fabs(provider->h2 - provider->h1) * 180.0 / M_PI, fabs(provider->w2 - provider->w1) * 180.0 / M_PI);
-	dbg("h = %3.2f ... %3.2f   (my %3.2f)\n", provider->h1 * 180.0 / M_PI, provider->h2 * 180.0 / M_PI, gses->myh * 180.0 / M_PI);
-	dbg("w = %3.2f ... %3.2f   (my %3.2f) \n", provider->w1 * 180.0 / M_PI, provider->w2 * 180.0 / M_PI, gses->myw * 180.0 / M_PI);
+	//rain_rainviewer_img2hw(provider, provider->hpx1, provider->wpx1+1, &provider->h1, &provider->w1);
+	//rain_rainviewer_img2hw(provider, provider->hpx2+1, provider->wpx2, &provider->h2, &provider->w2);
+	//dbg("dh = %3.2f,  dw = %3.2f \n", fabs(provider->h2 - provider->h1) * 180.0 / M_PI, fabs(provider->w2 - provider->w1) * 180.0 / M_PI);
+	//dbg("h = %3.2f ... %3.2f   (my %3.2f)\n", provider->h1 * 180.0 / M_PI, provider->h2 * 180.0 / M_PI, gses->myh * 180.0 / M_PI);
+	//dbg("w = %3.2f ... %3.2f   (my %3.2f) \n", provider->w1 * 180.0 / M_PI, provider->w2 * 180.0 / M_PI, gses->myw * 180.0 / M_PI);
 
 	provider->hmult = (provider->h2 - provider->h1) / (provider->hpx2 - provider->hpx1);
 	provider->wmult = (provider->w2 - provider->w1) / (provider->wpx2 - provider->wpx1);
 
-
+	provider->rv_pal = g_hash_table_new(g_direct_hash, g_direct_equal);
+	rain_pal_add(provider, 0x63615914, 0);
+	rain_pal_add(provider, 0x66635a19, 0);
+	rain_pal_add(provider, 0x69665c1e, 0);
+	rain_pal_add(provider, 0x6c685d24, 0);
+	rain_pal_add(provider, 0x6f6b5f29, 0);
+	rain_pal_add(provider, 0x726e612e, 0);
+	rain_pal_add(provider, 0x75706234, 0);
+	rain_pal_add(provider, 0x78736439, 0);
+	rain_pal_add(provider, 0x7c75653e, 0);
+	rain_pal_add(provider, 0x7f786744, 0);
+	rain_pal_add(provider, 0x827b6949, 0);
+	rain_pal_add(provider, 0x857d6a4e, 0);
+	rain_pal_add(provider, 0x88806c54, 0);
+	rain_pal_add(provider, 0x8b826d59, 0);
+	rain_pal_add(provider, 0x8e856f5e, 0);
+	rain_pal_add(provider, 0x92887164, 0);
+	rain_pal_add(provider, 0x9e93756e, 1);
+	rain_pal_add(provider, 0xaa9e7978, 1);
+	rain_pal_add(provider, 0xb6a97e82, 1);
+	rain_pal_add(provider, 0xc2b4828c, 1);
+	rain_pal_add(provider, 0xcec08796, 2);
+	rain_pal_add(provider, 0xd2c48ba0, 2);
+	rain_pal_add(provider, 0xd6c88faa, 2);
+	rain_pal_add(provider, 0xdacc93b4, 2);
+	rain_pal_add(provider, 0xded097be, 3);
+	rain_pal_add(provider, 0x88ddeeff, 3);
+	rain_pal_add(provider, 0x6cd1ebff, 3);
+	rain_pal_add(provider, 0x51c5e8ff, 3);
+	rain_pal_add(provider, 0x36bae5ff, 4);
+	rain_pal_add(provider, 0x1baee2ff, 4);
+	rain_pal_add(provider, 0x00a3e0ff, 4);
+	rain_pal_add(provider, 0x009ad5ff, 4);
+	rain_pal_add(provider, 0x0091caff, 5);
+	rain_pal_add(provider, 0x0088bfff, 5);
+	rain_pal_add(provider, 0x007fb4ff, 5);
+	rain_pal_add(provider, 0x0077aaff, 5);
+	rain_pal_add(provider, 0x0070a3ff, 6);
+	rain_pal_add(provider, 0x00699cff, 6);
+	rain_pal_add(provider, 0x006295ff, 6);
+	rain_pal_add(provider, 0x005b8eff, 6);
+	rain_pal_add(provider, 0x005588ff, 7);
+	rain_pal_add(provider, 0x005180ff, 7);
+	rain_pal_add(provider, 0x004e78ff, 7);
+	rain_pal_add(provider, 0x004a70ff, 7);
+	rain_pal_add(provider, 0x004768ff, 8);
+	rain_pal_add(provider, 0xffee00ff, 8);
+	rain_pal_add(provider, 0xffe000ff, 8);
+	rain_pal_add(provider, 0xffd200ff, 8);
+	rain_pal_add(provider, 0xffc500ff, 9);
+	rain_pal_add(provider, 0xffb700ff, 9);
+	rain_pal_add(provider, 0xffaa00ff, 9);
+	rain_pal_add(provider, 0xff9f00ff, 9);
+	rain_pal_add(provider, 0xff9500ff, 10);
+	rain_pal_add(provider, 0xff8b00ff, 10);
+	rain_pal_add(provider, 0xff8100ff, 10);
+	rain_pal_add(provider, 0xff4400ff, 10);
+	rain_pal_add(provider, 0xf23600ff, 11);
+	rain_pal_add(provider, 0xe62800ff, 11);
+	rain_pal_add(provider, 0xd91b00ff, 11);
+	rain_pal_add(provider, 0xcd0d00ff, 11);
+	rain_pal_add(provider, 0xc10000ff, 12);
+	rain_pal_add(provider, 0xa80000ff, 12);
+	rain_pal_add(provider, 0x8f0000ff, 12);
+	rain_pal_add(provider, 0x760000ff, 12);
+	rain_pal_add(provider, 0x5d0000ff, 13);
+	rain_pal_add(provider, 0xffaaffff, 13);
+	rain_pal_add(provider, 0xff9fffff, 13);
+	rain_pal_add(provider, 0xff95ffff, 13);
+	rain_pal_add(provider, 0xff8bffff, 14);
+	rain_pal_add(provider, 0xff81ffff, 14);
+	rain_pal_add(provider, 0xff77ffff, 14);
+	rain_pal_add(provider, 0xff6cffff, 14);
+	rain_pal_add(provider, 0xff62ffff, 14);
+	rain_pal_add(provider, 0xff58ffff, 14);
+	rain_pal_add(provider, 0xff4effff, 14);
+	rain_pal_add(provider, 0xffffffff, 14);
 
 }
 
-int tile_googlezoom(double dh, double dw)
-{
-	double hexp, wexp, gzh, gzw;
-	int gz;
+void rain_pal_add(struct rain_provider* provider, unsigned int rgba, int ci) {
+	g_hash_table_insert(provider->rv_pal, GUINT_TO_POINTER(rgba), GINT_TO_POINTER(ci));
 
-	hexp = 2 * M_PI / fabs(dh);
-	if (hexp < 2.0)
-		gzh = 0;
-	else
-		gzh = log(hexp) / log(2);
+}
+
+int tile_rainviewer_zoom(double dh, double dw)
+{
+	//double hexp, wexp, gzh, gzw;
+	//int gz;
+
+	//hexp = 2 * M_PI / fabs(dh);
+	//if (hexp < 2.0)
+	//	gzh = 0;
+	//else
+	//	gzh = log(hexp) / log(2);
 	//dbg("hexp=%4.2f  gzh=%4.2f \n", hexp, gzh);
 
 	//wexp = M_PI / fabs(dw);
-	wexp = ((2 * MAXLAT) * M_PI) / fabs(dw);
-	if (wexp < 2.0)
-		gzw = 0;
-	else
-		gzw = log(wexp) / log(2);
+	//wexp = ((2 * MAXLAT) * M_PI) / fabs(dw);
+	//if (wexp < 2.0)
+	//	gzw = 0;
+	//else
+	//	gzw = log(wexp) / log(2);
 	//dbg("wexp=%4.2f  gzw=%4.2f \n", wexp, gzw);
 
-	gz = (int)floor(Z_MIN(gzw, gzh));
+	//gz = (int)floor(Z_MIN(gzw, gzh));
 	//dbg("gz=%d\n", gz);
-	if (gz > 20) gz = 20;
+	//if (gz < 8) gz = 8; // rainviewer API limit is 7
+	//if (gz > 20) gz = 20;
+	return 5;
 
-	return (int)gz;
+	//return (int)gz;
 }
 
 void rain_rainviewer_timer(void *arg){
@@ -1308,32 +1451,34 @@ void rain_rainviewer_process_json(struct rain_provider *provider, const char *js
 		g_free(provider->rv_path);
 		provider->rv_path = s; 
 	}
-	char *host = zjson0_get_str(json, len, "host");
+
+	g_free(provider->rv_host);
+	provider->rv_host = zjson0_get_str(json, len, "host");
 	struct tm tm;
 	gmtime_r(&tt, &tm);
 	//dbg("rainviewer latest = %lld = %02d:%02dZ\n", (long long)tt,  tm.tm_hour, tm.tm_min);
 
-	if (host != NULL && (provider->rv_oldpath == NULL || strcmp(provider->rv_path, provider->rv_oldpath) != 0)){
+	if (provider->rv_host != NULL && (provider->rv_oldpath == NULL || strcmp(provider->rv_path, provider->rv_oldpath) != 0)){
 		
 		g_free(provider->title);
 		provider->title = g_strdup_printf("RainV. %02d:%02dZ", tm.tm_hour, tm.tm_min);
-
-
-		char *url = g_strdup_printf("%s%s/%d/%d/%6.4f/%6.4f/0/0_0.png", host, provider->rv_path, provider->rv_size, provider->rv_zoom, gses->myw * 180.0 / M_PI, gses->myh * 180.0 / M_PI);
-		//url = g_strdup("https://tile.openstreetmap.org/0/0/0.png");
-		//url = g_strdup("https://tile.openstreetmap.org/5/17/10.png");
-
-		provider->http = zhttp_init();
-		dbg("rainviewer %02d:%02dZ = '%s'\n", tm.tm_hour, tm.tm_min, url);
-		zhttp_get(provider->http, zsel, url, rain_rainviewer_downloaded_image, provider);
-		g_free(url);
+		provider->tileidx = 0;
+		rain_rainviewer_get_image(provider);
 	}
-    g_free(host);
 	g_free(provider->rv_oldpath);
 	provider->rv_oldpath = g_strdup(provider->rv_path);
-
 }
 
+void rain_rainviewer_get_image(struct rain_provider *provider) {
+	struct map_tile* tile = g_ptr_array_index(provider->tiles, provider->tileidx);
+	char* url = g_strdup_printf("%s%s%s", provider->rv_host, provider->rv_path, tile->url);
+	dbg("GET%02d '%s'\n", provider->tileidx, url);
+	provider->http = zhttp_init();
+	zhttp_get(provider->http, zsel, url, rain_rainviewer_downloaded_image, provider);
+	g_free(url);
+}
+
+
 void rain_rainviewer_downloaded_image(struct zhttp *http){
 	struct rain_provider *provider = (struct rain_provider*)http->arg;
 	SDL_Surface *newimg;
@@ -1355,9 +1500,13 @@ void rain_rainviewer_downloaded_image(struct zhttp *http){
 	}
 
 	if (http_is_content_type(http, "image/png")){
-		dbg("rain_rainviewer_downloaded_image: got %d bytes\n", http->response->len - http->dataofs);
+		dbg("rain_rainviewer_downloaded_image %d: got %d bytes\n", provider->tileidx, http->response->len - http->dataofs);
 
-		if (provider->rain->debug) zbinbuf_write_to_file(http->response, "_rainviewer.png", http->dataofs, http->response->len - http->dataofs);
+		if (provider->rain->debug) {
+			char filename[256];
+			g_snprintf(filename, sizeof(filename), "_rainviewer_%d.png", provider->tileidx);
+			zbinbuf_write_to_file(http->response, filename, http->dataofs, http->response->len - http->dataofs);
+		}
 
 
 		newimg = zpng_create(http->response->buf + http->dataofs, http->response->len - http->dataofs);
@@ -1368,49 +1517,71 @@ void rain_rainviewer_downloaded_image(struct zhttp *http){
 			provider->rv_oldpath = NULL;
 		}
 		else{
-			dbg("rainviewer bitmap loaded, %dx%d\n", newimg->w, newimg->h);
-			if (provider->img != NULL) {
-				SDL_FreeSurface(provider->img);
+			//dbg("rainviewer bitmap loaded, %dx%d\n", newimg->w, newimg->h);
+			struct map_tile* tile = (struct map_tile*)g_ptr_array_index(provider->tiles, provider->tileidx);
+
+			if (tile->img != NULL) {
+				SDL_FreeSurface(tile->img);
 			}
-			provider->img = newimg;
+			tile->img = newimg;
 			provider->pal = provider->rain->colors;
-			rain_rainviewer_load(provider);
-			rain_analyze_scps(provider->rain);
+
+			if (provider->tileidx == 0) {
+				int i;
+				for (i = 0; i < RAIN_COLORS; i++) {
+					GHashTable* hash = provider->hashes[i];
+					g_hash_table_foreach(hash, rain_truncate_hash, NULL);
+				}
+			}
+
+			rain_rainviewer_load(provider, tile);
+
+			if (provider->tileidx == provider->tiles->len - 1) {
+				rain_analyze_scps(provider->rain);
+			}
 		}
 		zhttp_free(http);
 		provider->http = NULL;
 		sw_set_gdirty(SWT_MAP);
 	}
+
+	provider->tileidx++;
+	if (provider->tileidx < provider->tiles->len) {
+		rain_rainviewer_get_image(provider);
+		return;
+	}
+
 	g_assert(provider->timer_id == 0);
 	provider->timer_id = zselect_timer_new(zsel, 1 * 60 * 1000, rain_rainviewer_timer, provider);
 
 }
 
 
-void rain_rainviewer_load(struct rain_provider *provider){
-	int ix, iy, c, ci, i;
-	SDL_Surface *img = provider->img;
+void rain_rainviewer_load(struct rain_provider *provider, struct map_tile* tile){
+	int ix, iy;
+	SDL_Surface *img = tile->img;
 
-	provider->w = img->w;
-	provider->h = img->h;
-
-	for (i = 0; i < RAIN_COLORS; i++) {
-		GHashTable *hash = provider->hashes[i];
-		g_hash_table_foreach(hash, rain_truncate_hash, NULL);
-	}
+	//provider->w = img->w;
+	//provider->h = img->h;
 
 
 	for (iy = 0; iy < img->h; iy++){
 		for (ix = 0; ix < img->w; ix++){
-			c = z_getpixel_fmt(img, ix, iy, sdl->screen->format);
+			int c = z_getpixel(img, ix, iy);
+			if (c == 0) continue;
+			int host = ntohl(c);
+			gpointer orig_key, orig_value;
 
-			int red = z_r(img, c);
-			ci = (red - 36) / 4;
-			//if (ci < 0) continue;
+			bool found = g_hash_table_lookup_extended(provider->rv_pal, GUINT_TO_POINTER(host), &orig_key, &orig_value);
+			if (!found) {
+				dbg("rainviewer: unknown color 0x%08x at [%d,%d]\n", host, ix, iy);
+				continue;
+			}
+
+			int ci = GPOINTER_TO_INT(orig_value);
 			if (ci < 5) continue;
-			if (ci >= RAIN_COLORS) ci = RAIN_COLORS - 1;
 
-			rain_provider_add(provider, provider->rv_px0 + ix, provider->rv_py0 + iy, ci/*5*/);
+			rain_provider_add(provider, tile, ix, iy, ci/*5*/);
 		}
 	}
 }
@@ -1424,14 +1595,16 @@ gpointer rain_rainviewer_hash(short kx, short ky, int ci){
 
 // y = (ln(tan(pi/4 + (x*pi/180)/2)))  * 90/pi for x = -85 .. 85
 
-void rain_rainviewer_hw2himg(struct rain_provider *provider, double h, double w, int *x, int *y){
-	*x = provider->rv_mul * (h + M_PI) / (2 * M_PI);
-	*y = provider->rv_mul * (M_PI - log(tan(M_PI_4 + w / 2))) / (2 * M_PI);
-}
+//void rain_rainviewer_hw2himg(struct rain_provider *provider, double h, double w, int *x, int *y){
+//	*x = provider->rv_mul * (h + M_PI) / (2 * M_PI);
+//	*y = provider->rv_mul * (M_PI - log(tan(M_PI_4 + w / 2))) / (2 * M_PI);
+//}
 
-void rain_rainviewer_img2hw(struct rain_provider *provider, float x, float y, double *h, double *w){
-	*h = (x * 2 * M_PI / provider->rv_mul) - M_PI;
-	*w = 2 * (atan(exp(M_PI - y * 2 * M_PI / provider->rv_mul)) - M_PI_4);
+void rain_rainviewer_img2hw(struct rain_provider *provider, struct map_tile *tile, int x, int y, double *h, double *w){
+	double lon = tiles_tileX2lon(tile->x, x, provider->rv_zoom);
+	double lat = tiles_tileY2lat(tile->y, y, provider->rv_zoom);
+	*h = lon * M_PI / 180.0;
+	*w = lat * M_PI / 180.0;
 }
 
 
