@@ -29,7 +29,11 @@ struct rain_km{
 	float kx4, ky4;	  // right lower
 };
 
-#define RV_TILES 16
+struct map_tile{
+	int x, y;
+	char url[256];
+	SDL_Surface* img;
+};
 
 struct rain_provider{
 	char *name; // const not freeed
@@ -45,10 +49,10 @@ struct rain_provider{
 	struct rain *rain;
 	int timer_id;
     struct zhttp *http;
-	int w, h;
+	//int w22, h22;
 	GHashTable *hashes[RAIN_COLORS]; // of struct zbinbuf of struct rain_pixel
 	gpointer (*hash_func)(short kx, short ky, int ci);
-	void(*img2hw)(struct rain_provider *provider, float ix, float iy, double *h, double *w);
+	void(*img2hw)(struct rain_provider *provider, struct map_tile *tile, int ix, int iy, double *h, double *w);
 
 	int mask, step;
 	double hmult, wmult;
@@ -56,10 +60,14 @@ struct rain_provider{
 
 	int ofs_min;
 
-	int rv_zoom, rv_size, rv_pow, rv_mul;
+	int rv_zoom, rv_size/*, rv_pow, rv_mul*/;
+	GPtrArray* tiles; // of struct rv_tile
+	int tileidx;
+	char* rv_host;
 	char *rv_path, *rv_oldpath;
-	int rv_mypx, rv_mypy;
-	float rv_px0, rv_py0;
+	GHashTable* rv_pal;
+	//int rv_mypx, rv_mypy;
+	//float rv_px0, rv_py0;
 
 };
 
@@ -100,7 +108,7 @@ void rain_add_scps(struct rain_provider *prov);
 int rain_scp_exists(struct rain_provider *prov, float kx, float ky);
 
 void free_rain_provider(struct rain_provider *provider);
-void rain_linear_img2hw(struct rain_provider *provider, float ix, float iy, double *h, double *w);
+void rain_linear_img2hw(struct rain_provider *provider, struct map_tile *tile, int ix, int iy, double *h, double *w);
 void rain_reload(void);
 
 void rain_meteox_timer(void *arg);
@@ -122,18 +130,25 @@ void rain_weatheronline_timer(void *arg);
 void rain_weatheronline_downloaded(struct zhttp *http);
 void rain_weatheronline_load(struct rain_provider *provider);
 gpointer rain_weatheronline_hash(short px, short py, int color);
-void rain_weatheronline_img2hw(struct rain_provider *provider, float ix, float iy, double *h, double *w);
+void rain_weatheronline_img2hw(struct rain_provider *provider, struct map_tile* tile, int ix, int iy, double *h, double *w);
+
+int tiles_lon2tileX(double lon, int zoom);
+int tiles_lat2tileY(double lat, int zoom);
+double tiles_tileX2lon(int tileX, int ix, int zoom);
+double tiles_tileY2lat(int tileY, int iy, int zoom);
 
 void rain_rainviewer_init(struct rain_provider *provider);
-int tile_googlezoom(double dh, double dw);
+void rain_pal_add(struct rain_provider* provider, unsigned int rgba, int ci);
+int tile_rainviewer_zoom(double dh, double dw);
 void rain_rainviewer_timer(void *arg);
 void rain_rainviewer_downloaded_json(struct zhttp *http);
 void rain_rainviewer_process_json(struct rain_provider*, const char *json);
+void rain_rainviewer_get_image(struct rain_provider* provider);
 void rain_rainviewer_downloaded_image(struct zhttp *http);
-void rain_rainviewer_load(struct rain_provider *provider);
+void rain_rainviewer_load(struct rain_provider *provider, struct map_tile* tile);
 gpointer rain_rainviewer_hash(short kx, short ky, int ci);
-void rain_rainviewer_hw2himg(struct rain_provider *provider, double h, double w, int *x, int *y);
-void rain_rainviewer_img2hw(struct rain_provider *provider, float x, float y, double *h, double *w);
+//void rain_rainviewer_hw2himg(struct rain_provider *provider, double h, double w, int *x, int *y);
+void rain_rainviewer_img2hw(struct rain_provider *provider, struct map_tile* tile, int x, int y, double *h, double *w);
 
 
 
