@@ -812,7 +812,7 @@ void sw_sdr_redraw(struct subwin *sw, struct band *band, int flags){
 
 
 gpointer sdr_thread_func(gpointer data){
-    int total = 0, i;
+    int i;
     struct sdr *sdr = (struct sdr *)data;
 	struct timeval tv1, tv2, stop, tvstart;
 	long oldsec = 0;
@@ -910,8 +910,6 @@ gpointer sdr_thread_func(gpointer data){
 
 		gettimeofday(&tv2, NULL);
 
-        total += frames;
-        
 		b = buf;
 		pd = (double*)(sdr->iq + (sdr->n - sdr->frames));
 		for (i = 0; i < sdr->frames; i++){
