@@ -287,7 +287,6 @@ gpointer ssbd_play_thread_func(gpointer data){
     int frames, samples, writtenf, writtens;
     int err;
     short *wrptr;
-    int xxx = -1;
 	sf_count_t cnt;
 	int tocopy; // in samples
     
@@ -303,7 +302,6 @@ gpointer ssbd_play_thread_func(gpointer data){
     while(1){
 		int seek;
 
-        xxx++;
         if (gssbd->proc_break) {
             gssbd->dsp->reset(gssbd->dsp);
             goto x; 
